# -*- coding: UTF-8 -*-

import sys, os, errno
from artix.common import logger

"""
Объект отвечающий за файл блокировки
В конструктор передается имя файла блокировки
"""
class ServiceLock(object):
    def __init__(self, filename, logger):
        self.lockFileName = filename
        self.log = logger

    """
    Создает файл блокировки. Если файл уже есть и процесс с этим pid'ом запущен,
    завершаем работу.
    @param exitOnLock если true и сервис уже запущен, выполнять выход
    @return true если блокировка выполнена
    """
    def lock(self, exitOnLock = True):
        if os.path.exists(self.lockFileName):
            self.log.info(u'Найден файл блокировки %s' % self.lockFileName)
            with open(self.lockFileName, 'r') as lockFile:
                pid = int(lockFile.read())
            if self.pid_exists(pid):
                self.log.error(u'Сервис уже запущен. pid = %s' % pid)
                if exitOnLock:
                    os._exit(1) # выход без выполнения atexit
                return False
            else:
                self.log.warn(u'Предыдущая сессия была завершена с ошибкой')
        if not os.path.exists(os.path.dirname(self.lockFileName)):
            os.makedirs(os.path.dirname(self.lockFileName))
        self.log.info(u'Создание файла блокировки %s' % self.lockFileName)
        with open(self.lockFileName, 'w') as lockFile:
            lockFile.write(str(os.getpid()))
        return True

    """
    Удаляет файл блокировки
    """
    def unlock(self, signalNumber = None, stackFrame = None):
        # Параметры, избавляющие от ошибки по количеству параметров, для случая использования в качестве обработчика сигналов
        del stackFrame
        if os.path.exists(self.lockFileName):
            self.log.info(u'Удаление файла блокировки %s' % self.lockFileName)
            try:
                os.remove(self.lockFileName)
            except OSError as e:
                self.log.info(u'Ошибка удаления файла: %s' % e.strerror)
        if signalNumber is not None:
            os._exit(1)

    """
    Возвращает true если процесс с указанным pid запущен
    """
    def pid_exists(self, pid):
        try:
            os.kill(pid, 0)
        except OSError as e:
            return e.errno == errno.EPERM
        else:
            return True
